use crate::{IRect, Image, Tileset};
use glam::IVec2;
use std::collections::VecDeque;

pub(crate) struct Console {
    lines: VecDeque<String>,
    max_lines: usize,
    max_line_length: usize,
}

impl Console {
    pub(crate) fn new(max_lines: i32, max_line_length: i32) -> Self {
        Self {
            lines: VecDeque::new(),
            max_lines: max_lines as usize,
            max_line_length: max_line_length as usize,
        }
    }

    pub(crate) fn add(&mut self, str: &str) {
        if str.len() > self.max_line_length {
            let (first, rest) = str.split_at(self.max_line_length);
            self.add(first);
            self.add(rest);
        } else {
            self.lines.push_back(str.into());
            while self.lines.len() > self.max_lines {
                self.lines.pop_front();
            }
        }
    }

    pub(crate) fn draw_to_image(&self, target: &mut Image, pos: IVec2, font: &Tileset) {
        let height = self.max_lines as i32 * font.tile_size().y;
        let width = self.max_line_length as i32 * font.tile_size().x;
        target.fill_rect(
            IRect {
                pos,
                size: IVec2 {
                    x: width,
                    y: height,
                },
            },
            254,
        );
        let y = height + pos.y;
        target.draw_line(
            IVec2 { x: pos.x, y },
            IVec2 {
                x: pos.x + width,
                y,
            },
            253,
        );
        for (i, str) in self.lines.iter().enumerate() {
            target.draw_string(
                pos + IVec2 {
                    x: 0,
                    y: i as i32 * font.tile_size().y,
                },
                str,
                font,
            );
        }
    }
}
