use crate::vec_util::IVec2Helper;
use crate::{IRect, Image};
use glam::IVec2;
use std::fs;
use std::rc::Rc;

pub struct Tileset {
    count: i32,
    size: IVec2,
    images: Vec<Image>,
}

impl Tileset {
    pub fn load(path: &str, tile_count: IVec2) -> Rc<Self> {
        Self::load_data(fs::read(path).unwrap().as_slice(), tile_count)
    }

    pub fn load_data(data: &[u8], tile_count: IVec2) -> Rc<Self> {
        let img = Image::load_data(data);
        let mut images: Vec<Image> = vec![];
        let size = IVec2 {
            x: img.size().x / tile_count.x,
            y: img.size().y / tile_count.y,
        };
        for tile in tile_count.to_rect().iter() {
            let mut image = Image::new(size);
            image.draw_image_partial(
                IVec2::ZERO,
                &img,
                IRect {
                    pos: IVec2 {
                        x: tile.x * size.x,
                        y: tile.y * size.y,
                    },
                    size,
                },
            );
            images.push(image);
        }
        Tileset {
            count: images.len() as i32,
            size,
            images,
        }
        .into()
    }

    pub fn get(&self, idx: i32) -> &Image {
        &self.images[idx as usize]
    }

    pub fn count(&self) -> i32 {
        self.count
    }

    pub fn tile_size(&self) -> IVec2 {
        self.size
    }
}
